@@ -9,7 +9,7 @@
 // Author:      Robin Dunn
 //
 // Created:     13-Jan-2000
-// RCS-ID:      $Id: ScintillaWX.cpp 54230 2008-06-15 11:45:05Z VZ $
+// RCS-ID:      $Id: ScintillaWX.cpp 55221 2008-08-24 04:06:41Z RD $
 // Copyright:   (c) 2000 by Total Control Software
 // Licence:     wxWindows license
 /////////////////////////////////////////////////////////////////////////////
@@ -682,11 +682,15 @@ bool ScintillaWX::DestroySystemCaret() {
 //----------------------------------------------------------------------
 
 
-long ScintillaWX::DefWndProc(unsigned int /*iMessage*/, unsigned long /*wParam*/, long /*lParam*/) {
+sptr_t ScintillaWX::DefWndProc(unsigned int /*iMessage*/,
+                               uptr_t /*wParam*/,
+                               sptr_t /*lParam*/) {
     return 0;
 }
 
-long ScintillaWX::WndProc(unsigned int iMessage, unsigned long wParam, long lParam) {
+sptr_t ScintillaWX::WndProc(unsigned int iMessage,
+                            uptr_t wParam,
+                            sptr_t lParam) {
       switch (iMessage) {
       case SCI_CALLTIPSHOW: {
           // NOTE: This is copied here from scintilla/src/ScintillaBase.cxx
