@@ -2,7 +2,7 @@
 // Name:        src/gtk/menu.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: menu.cpp 54173 2008-06-12 20:25:44Z JS $
+// Id:          $Id: menu.cpp 54676 2008-07-18 02:45:48Z PC $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -1145,18 +1145,9 @@ bool wxMenu::GtkAppend(wxMenuItem *mitem, int pos)
             wxASSERT_MSG( mitem->GetKind() == wxITEM_NORMAL,
                             _T("only normal menu items can have bitmaps") );
 
-            if ( bitmap.HasPixbuf() )
-            {
-                image = gtk_image_new_from_pixbuf(bitmap.GetPixbuf());
-            }
-            else
-            {
-                GdkPixmap *gdk_pixmap = bitmap.GetPixmap();
-                GdkBitmap *gdk_bitmap = bitmap.GetMask() ?
-                                            bitmap.GetMask()->GetBitmap() :
-                                            (GdkBitmap*) NULL;
-                image = gtk_image_new_from_pixmap( gdk_pixmap, gdk_bitmap );
-            }
+            // always use pixbuf, because pixmap mask does not
+            // work with disabled images in some themes
+            image = gtk_image_new_from_pixbuf(bitmap.GetPixbuf());
         }
 
         if ( image )
