@@ -2,7 +2,7 @@
 // Name:        src/gtk/radiobox.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: radiobox.cpp,v 1.130.4.1 2007/03/25 02:03:07 VZ Exp $
+// Id:          $Id: radiobox.cpp 49343 2007-10-23 07:27:12Z PC $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -49,7 +49,7 @@ public:
 //-----------------------------------------------------------------------------
 
 #include "wx/listimpl.cpp"
-WX_DEFINE_LIST( wxRadioBoxButtonsInfoList );
+WX_DEFINE_LIST(wxRadioBoxButtonsInfoList)
 
 extern bool          g_blockEventsOnDrag;
 
