@@ -2,7 +2,7 @@
 // Name:        src/html/htmlwin.cpp
 // Purpose:     wxHtmlWindow class for parsing & displaying HTML (implementation)
 // Author:      Vaclav Slavik
-// RCS-ID:      $Id: htmlwin.cpp,v 1.129 2006/10/30 16:18:32 VZ Exp $
+// RCS-ID:      $Id: htmlwin.cpp 52186 2008-02-28 22:50:25Z VS $
 // Copyright:   (c) 1999 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -533,6 +533,7 @@ bool wxHtmlWindow::LoadPage(const wxString& location)
         {
             wxLogError(_("Unable to open requested HTML document: %s"), location.c_str());
             m_tmpCanDrawLocks--;
+            SetHTMLStatusText(wxEmptyString);
             return false;
         }
 
@@ -953,6 +954,7 @@ bool wxHtmlWindow::CopySelection(ClipboardType t)
 void wxHtmlWindow::OnLinkClicked(const wxHtmlLinkInfo& link)
 {
     wxHtmlLinkEvent event(GetId(), link);
+    event.SetEventObject(this);
     if (!GetEventHandler()->ProcessEvent(event))
     {
         // the default behaviour is to load the URL in this window
@@ -1148,9 +1150,13 @@ void wxHtmlWindow::OnMouseUp(wxMouseEvent& event)
         ReleaseMouse();
         m_makingSelection = false;
 
-        // did the user move the mouse far enough from starting point?
-        if ( CopySelection(Primary) )
+        // if m_selection=NULL, the user didn't move the mouse far enough from
+        // starting point and the mouse up event is part of a click, the user
+        // is not selecting text:
+        if ( m_selection )
         {
+            CopySelection(Primary);
+
             // we don't want mouse up event that ended selecting to be
             // handled as mouse click and e.g. follow hyperlink:
             return;
