@@ -4,7 +4,7 @@
 // Author:      Stefan Csomor
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: graphics.cpp 49306 2007-10-21 18:16:07Z SC $
+// RCS-ID:      $Id: graphics.cpp 54918 2008-08-02 06:31:46Z SC $
 // Copyright:   (c) Stefan Csomor
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -23,6 +23,7 @@
     #include "wx/image.h"
 #endif
 
+#include "wx/strconv.h"
 #include "wx/mac/uma.h"
 
 #ifdef __MSL__
@@ -728,11 +729,20 @@ public:
     ~wxMacCoreGraphicsFontData();
 
     virtual ATSUStyle GetATSUStyle() { return m_macATSUIStyle; }
+#if !wxUSE_UNICODE
+    const wxMBConv& GetConverter() const { return m_conv; }
+#endif
 private :
     ATSUStyle m_macATSUIStyle;
+#if !wxUSE_UNICODE
+    wxCSConv m_conv;
+#endif
 };
 
 wxMacCoreGraphicsFontData::wxMacCoreGraphicsFontData(wxGraphicsRenderer* renderer, const wxFont &font, const wxColour& col) : wxGraphicsObjectRefData( renderer )
+#if !wxUSE_UNICODE
+    , m_conv( font.GetEncoding() == wxFONTENCODING_DEFAULT ? wxFONTENCODING_SYSTEM : font.GetEncoding() )
+#endif
 {
     m_macATSUIStyle = NULL;
 
@@ -1335,9 +1345,6 @@ void wxMacCoreGraphicsContext::Init()
     m_cgContext = NULL;
     m_releaseContext = false;
     m_windowRef = NULL;
-
-    HIRect r = CGRectMake(0,0,0,0);
-    m_clipRgn.Set(HIShapeCreateWithRect(&r));
 }
 
 wxMacCoreGraphicsContext::wxMacCoreGraphicsContext( wxGraphicsRenderer* renderer, CGContextRef cgcontext ) : wxGraphicsContext(renderer)
@@ -1398,15 +1405,24 @@ void wxMacCoreGraphicsContext::EnsureIsValid()
         CGContextConcatCTM( m_cgContext, m_windowTransform );
 		CGContextSaveGState( m_cgContext );
 		m_releaseContext = true;
-		if ( !HIShapeIsEmpty(m_clipRgn) )
+		if ( (HIShapeRef) m_clipRgn != NULL )
 		{
             // the clip region is in device coordinates, so we convert this again to user coordinates
             wxMacCFRefHolder<HIMutableShapeRef> hishape ;
             hishape.Set( HIShapeCreateMutableCopy( m_clipRgn ) );
             CGPoint transformedOrigin = CGPointApplyAffineTransform( CGPointZero,m_windowTransform);
             HIShapeOffset( hishape, -transformedOrigin.x, -transformedOrigin.y );
-			HIShapeReplacePathInCGContext( hishape, m_cgContext );
-			CGContextClip( m_cgContext );
+            // if the shape is empty, HIShapeReplacePathInCGContext doesn't work
+            if ( HIShapeIsEmpty(hishape))
+            {
+                CGRect empty = CGRectMake( 0,0,0,0 );
+                CGContextClipToRect( m_cgContext, empty );
+            }
+            else
+            {
+                HIShapeReplacePathInCGContext( hishape, m_cgContext );
+                CGContextClip( m_cgContext );
+            }
 		}
 		CGContextSaveGState( m_cgContext );
 	}
@@ -1454,10 +1470,19 @@ void wxMacCoreGraphicsContext::Clip( const wxRegion &region )
 {
     if( m_cgContext )
     {
-        HIShapeRef shape = HIShapeCreateWithQDRgn( (RgnHandle) region.GetWXHRGN() );
-        HIShapeReplacePathInCGContext( shape, m_cgContext );
-        CGContextClip( m_cgContext );
-        CFRelease( shape );
+        wxMacCFRefHolder<HIShapeRef> shape;
+        shape = HIShapeCreateWithQDRgn( (RgnHandle) region.GetWXHRGN() );
+        // if the shape is empty, HIShapeReplacePathInCGContext doesn't work
+        if ( HIShapeIsEmpty(shape))
+        {
+            CGRect empty = CGRectMake( 0,0,0,0 );
+            CGContextClipToRect( m_cgContext, empty );
+        }
+        else
+        {
+            HIShapeReplacePathInCGContext( shape, m_cgContext );
+            CGContextClip( m_cgContext );
+        }
     }
     else
     {
@@ -1507,8 +1532,7 @@ void wxMacCoreGraphicsContext::ResetClip()
     }
     else
     {
-        HIRect r = CGRectMake(0,0,0,0);
-        m_clipRgn.Set(HIShapeCreateWithRect(&r));
+        m_clipRgn.Release();
     }
 }
 
@@ -1734,8 +1758,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 
 void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDouble y, wxDouble angle )
 {
-    if ( m_font.IsNull() )
-        return;
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::DrawText - no valid font set") );
 
     EnsureIsValid();
 
@@ -1751,7 +1774,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , str.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1761,7 +1784,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 #if wxUSE_UNICODE
     ubuf = (UniChar*) str.wc_str();
 #else
-    wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
@@ -1849,7 +1872,7 @@ void wxMacCoreGraphicsContext::DrawText( const wxString &str, wxDouble x, wxDoub
 void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *width, wxDouble *height,
                             wxDouble *descent, wxDouble *externalLeading ) const
 {
-    wxCHECK_RET( !m_font.IsNull(), wxT("wxDC(cg)::DoGetTextExtent - no valid font set") );
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::GetTextExtent - no valid font set") );
 
     if ( width )
         *width = 0;
@@ -1876,7 +1899,7 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , str.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1886,7 +1909,7 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
 #if wxUSE_UNICODE
     ubuf = (UniChar*) str.wc_str();
 #else
-    wxWCharBuffer wchar = str.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = str.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
@@ -1898,6 +1921,9 @@ void wxMacCoreGraphicsContext::GetTextExtent( const wxString &str, wxDouble *wid
 
     wxASSERT_MSG( status == noErr , wxT("couldn't create the layout of the text") );
 
+    status = ::ATSUSetTransientFontMatching( atsuLayout , true );
+    wxASSERT_MSG( status == noErr , wxT("couldn't setup transient font matching") );
+
     ATSUTextMeasurement textBefore, textAfter;
     ATSUTextMeasurement textAscent, textDescent;
 
@@ -1924,12 +1950,15 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
     widths.Empty();
     widths.Add(0, text.length());
 
+    wxCHECK_RET( !m_font.IsNull(), wxT("wxMacCoreGraphicsContext::GetPartialTextExtents - no valid font set") );
+
     if (text.empty())
         return;
 
     ATSUTextLayout atsuLayout;
     UniCharCount chars = text.length();
     UniChar* ubuf = NULL;
+    OSStatus status = noErr;
 
 #if SIZEOF_WCHAR_T == 4
     wxMBConvUTF16 converter;
@@ -1938,7 +1967,7 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , text.wc_str(), unicharlen + 2 );
 #else
-    const wxWCharBuffer wchar = text.wc_str( wxConvLocal );
+    const wxWCharBuffer wchar = text.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     size_t unicharlen = converter.WC2MB( NULL , wchar.data() , 0 );
     ubuf = (UniChar*) malloc( unicharlen + 2 );
     converter.WC2MB( (char*) ubuf , wchar.data() , unicharlen + 2 );
@@ -1948,16 +1977,22 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
 #if wxUSE_UNICODE
     ubuf = (UniChar*) text.wc_str();
 #else
-    wxWCharBuffer wchar = text.wc_str( wxConvLocal );
+    wxWCharBuffer wchar = text.wc_str( ((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetConverter() );
     chars = wxWcslen( wchar.data() );
     ubuf = (UniChar*) wchar.data();
 #endif
 #endif
 
     ATSUStyle style = (((wxMacCoreGraphicsFontData*)m_font.GetRefData())->GetATSUStyle());
-    ::ATSUCreateTextLayoutWithTextPtr( (UniCharArrayPtr) ubuf , 0 , chars , chars , 1 ,
+    status = ::ATSUCreateTextLayoutWithTextPtr( (UniCharArrayPtr) ubuf , 0 , chars , chars , 1 ,
         &chars , &style , &atsuLayout );
+    wxASSERT_MSG( status == noErr , wxT("couldn't create the layout of the text") );
 
+    status = ::ATSUSetTransientFontMatching( atsuLayout , true );
+    wxASSERT_MSG( status == noErr , wxT("couldn't setup transient font matching") );
+
+// new implementation from JS, keep old one just in case
+#if 0
     for ( int pos = 0; pos < (int)chars; pos ++ )
     {
         unsigned long actualNumberOfBounds = 0;
@@ -1973,6 +2008,31 @@ void wxMacCoreGraphicsContext::GetPartialTextExtents(const wxString& text, wxArr
         widths[pos] = FixedToInt( glyphBounds.upperRight.x - glyphBounds.upperLeft.x );
         //unsigned char uch = s[i];
     }
+#else
+    ATSLayoutRecord *layoutRecords = NULL;
+    ItemCount glyphCount = 0;
+    
+    // Get the glyph extents
+    OSStatus err = ::ATSUDirectGetLayoutDataArrayPtrFromTextLayout(atsuLayout,
+                                                                   0,
+                                                                   kATSUDirectDataLayoutRecordATSLayoutRecordCurrent,
+                                                                   (void **)
+                                                                   &layoutRecords,
+                                                                   &glyphCount);
+    wxASSERT(glyphCount == (text.length()+1));
+    
+    if ( err == noErr && glyphCount == (text.length()+1))
+    {
+        for ( int pos = 1; pos < (int)glyphCount ; pos ++ )
+        {
+            widths[pos-1] = FixedToInt( layoutRecords[pos].realPos );
+        }
+    }
+    
+    ::ATSUDirectReleaseLayoutDataArrayPtr(NULL,
+                                          kATSUDirectDataLayoutRecordATSLayoutRecordCurrent,
+                                          (void **) &layoutRecords);
+#endif
 
     ::ATSUDisposeTextLayout(atsuLayout);
 #if SIZEOF_WCHAR_T == 4
