@@ -3,7 +3,7 @@
 // Purpose:     XRC resources
 // Author:      Vaclav Slavik
 // Created:     2000/03/05
-// RCS-ID:      $Id: xmlres.cpp,v 1.104 2007/01/21 01:01:39 VZ Exp $
+// RCS-ID:      $Id: xmlres.cpp 51218 2008-01-14 22:40:28Z VS $
 // Copyright:   (c) 2000 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -942,7 +942,7 @@ wxString wxXmlResourceHandler::GetText(const wxString& param, bool translate)
 #else
             // The string is internally stored as UTF-8, we have to convert
             // it into system's default encoding so that it can be displayed:
-            return wxString(str2.mb_str(wxConvUTF8), wxConvLocal);
+            return wxString(str2.wc_str(wxConvUTF8), wxConvLocal);
 #endif
         }
     }
@@ -968,20 +968,18 @@ long wxXmlResourceHandler::GetLong(const wxString& param, long defaultv)
 
 float wxXmlResourceHandler::GetFloat(const wxString& param, float defaultv)
 {
+    wxString str = GetParamValue(param);
+
+    // strings in XRC always use C locale but wxString::ToDouble() uses the
+    // current one, so transform the string to it supposing that the only
+    // difference between them is the decimal separator
+    str.Replace(wxT("."), wxLocale::GetInfo(wxLOCALE_DECIMAL_POINT,
+                                            wxLOCALE_CAT_NUMBER));
+
     double value;
-    wxString str1 = GetParamValue(param);
-
-#ifndef __WXWINCE__
-    const char *prevlocale = setlocale(LC_NUMERIC, "C");
-#endif
-
-    if (!str1.ToDouble(&value))
+    if (!str.ToDouble(&value))
         value = defaultv;
 
-#ifndef __WXWINCE__
-    setlocale(LC_NUMERIC, prevlocale);
-#endif
-
     return wx_truncate_cast(float, value);
 }
 
@@ -1666,6 +1664,7 @@ static void AddStdXRCID_Records()
     stdID(wxID_HELP_CONTEXT);
     stdID(wxID_CLOSE_ALL);
     stdID(wxID_PREFERENCES);
+    stdID(wxID_EDIT);
     stdID(wxID_CUT);
     stdID(wxID_COPY);
     stdID(wxID_PASTE);
