@@ -11,17 +11,19 @@
 import os
 import webbrowser
 import wx
+import wx.lib.scrolledpanel as scrolled
 import wx.lib.platebtn as platebtn
 
 #-----------------------------------------------------------------------------#
 
-class TestPanel(wx.Panel):
+class TestPanel(scrolled.ScrolledPanel):
     def __init__(self, parent, log):
         self.log = log
-        wx.Panel.__init__(self, parent)
+        scrolled.ScrolledPanel.__init__(self, parent, size=(400, 400))
 
         # Layout
         self.__DoLayout()
+        self.SetupScrolling()
 
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
@@ -169,6 +171,17 @@ class TestPanel(wx.Panel):
         self.log.write("BUTTON CLICKED: Id: %d, Label: %s" % \
                        (evt.GetId(), evt.GetEventObject().LabelText))
 
+    def OnChildFocus(self, evt):
+        """Override ScrolledPanel.OnChildFocus to prevent erratic
+        scrolling on wxMac.
+
+        """
+        if wx.Platform != '__WXMAC__':
+            evt.Skip()
+
+        child = evt.GetWindow()
+        self.ScrollChildIntoView(child)
+
     def OnMenu(self, evt):
         """Events from button menus"""
         self.log.write("MENU SELECTED: %d" % evt.GetId())
