@@ -7,18 +7,17 @@
 # License: wxWindows License                                                  #
 ###############################################################################
 
-"""Configuration Dialog"""
+"""Launch Configuration Dialog"""
+
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: cfgdlg.py 53724 2008-05-23 15:54:50Z CJP $"
-__revision__ = "$Revision: 53724 $"
+__svnid__ = "$Id: cfgdlg.py 55064 2008-08-12 03:27:17Z CJP $"
+__revision__ = "$Revision: 55064 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
 import sys
 import wx
 import wx.lib.mixins.listctrl as listmix
-#import wx.lib.colourselect as colourselect
-import eclib.colorsetter as colorsetter
 import cStringIO
 import zlib
 
@@ -26,6 +25,8 @@ import zlib
 import handlers
 
 # Editra Libraries
+import eclib.colorsetter as colorsetter
+import eclib.elistmix as elistmix
 from profiler import Profile_Get, Profile_Set
 import ed_msg
 import util
@@ -125,12 +126,18 @@ class ConfigDialog(wx.Frame):
     def __DoLayout(self):
         """Layout the dialog"""
         sizer = wx.BoxSizer(wx.VERTICAL)
+
         hsizer = wx.BoxSizer(wx.HORIZONTAL)
-        noteb = wx.Notebook(self)
+        vsizer = wx.BoxSizer(wx.VERTICAL)
+        panel = wx.Panel(self)
+        noteb = wx.Notebook(panel)
         noteb.AddPage(ConfigPanel(noteb), _("General"))
         noteb.AddPage(MiscPanel(noteb), _("Misc"))
         hsizer.AddMany([((5, 5), 0), (noteb, 1, wx.EXPAND), ((5, 5), 0)])
-        sizer.AddMany([((5, 5), 0), (hsizer, 1, wx.EXPAND), ((10, 10), 0)])
+        vsizer.AddMany([((5, 5), 0), (hsizer, 1, wx.EXPAND), ((10, 10), 0)])
+        panel.SetSizer(vsizer)
+
+        sizer.Add(panel, 1, wx.EXPAND)
         self.SetSizer(sizer)
         self.SetAutoLayout(True)
         self.SetInitialSize()
@@ -374,6 +381,8 @@ class MiscPanel(wx.Panel):
     def __DoLayout(self):
         """Layout the controls"""
         msizer = wx.BoxSizer(wx.VERTICAL)
+        sbox = wx.StaticBox(self, label=_("Text Colors"))
+        boxsz = wx.StaticBoxSizer(sbox, wx.VERTICAL)
 
         # Launch Config
         cfg = Profile_Get(LAUNCH_PREFS, default=dict())
@@ -389,8 +398,6 @@ class MiscPanel(wx.Panel):
             cbtn = colorsetter.ColorSetter(self, btn[0], color=cfg.get(btn[1]))
             colors[btn[0]] = cbtn
 
-        sbox = wx.StaticBox(self, label=_("Text Colors"))
-        boxsz = wx.StaticBoxSizer(sbox, wx.VERTICAL)
         flexg = wx.FlexGridSizer(5, 5, 5, 5)
         flexg.AddGrowableCol(1, 1)
         flexg.AddGrowableCol(3, 1)
@@ -465,38 +472,14 @@ class MiscPanel(wx.Panel):
 
 class AutoWidthListCtrl(listmix.ListCtrlAutoWidthMixin,
                         listmix.TextEditMixin,
+                        elistmix.ListRowHighlighter,
                         wx.ListCtrl):
     """Auto-width adjusting list for showing editing the commands"""
     def __init__(self, *args, **kwargs):
         wx.ListCtrl.__init__(self, *args, **kwargs)
         listmix.ListCtrlAutoWidthMixin.__init__(self)
         listmix.TextEditMixin.__init__(self)
-
-    def Append(self, entry):
-        """Append an entry to the list
-        @param entry: entry to add
-
-        """
-        wx.ListCtrl.Append(self, entry)
-        self.ColorRow(self.GetItemCount() - 1)
-        
-    def ColorRow(self, index):
-        """Color the given index
-        @param index: list index
-
-        """
-        if index % 2:
-            color = wx.SystemSettings_GetColour(wx.SYS_COLOUR_HIGHLIGHT)
-            color = util.AdjustColour(color, 15)
-            self.SetItemBackgroundColour(index, color)
-
-    def SetStringItem(self, index, col, label, imageId=-1):
-        """Set the item in the list at the given index and colorize
-        the row if it is an odd one.
-
-        """
-        wx.ListCtrl.SetStringItem(self, index, col, label, imageId=imageId)
-        self.ColorRow(index)
+        elistmix.ListRowHighlighter.__init__(self)
 
 #-----------------------------------------------------------------------------#
 
