@@ -12,8 +12,8 @@ Provides helper functions and classes for managing documents and their services.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: doctools.py 52855 2008-03-27 14:53:06Z CJP $"
-__revision__ = "$Revision: 52855 $"
+__svnid__ = "$Id: doctools.py 54365 2008-06-25 23:46:11Z CJP $"
+__revision__ = "$Revision: 54365 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -85,7 +85,10 @@ class DocPositionMgr(object):
                 tfile = util.GetFileWriter(book)
                 tfile.close()
             except (IOError, OSError):
-                util.Log("[docpositionmgr][err] failed to load book")
+                util.Log("[docpositionmgr][err] failed to load book: %s" % book)
+                return False
+            except AttributeError:
+                util.Log("[docpositionmgr][err] Failed to create: %s" % book)
                 return False
 
         reader = util.GetFileReader(book, sys.getfilesystemencoding())
