@@ -14,8 +14,8 @@ access point for all common variables in the project.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_glob.py 54045 2008-06-09 02:24:02Z CJP $"
-__revision__ = "$Revision: 54045 $"
+__svnid__ = "$Id: ed_glob.py 55858 2008-09-25 12:50:12Z CJP $"
+__revision__ = "$Revision: 55858 $"
 
 __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_CLOSE', 'ID_CLOSEALL', 'ID_SAVE', 'ID_SAVEAS', 'ID_SAVEALL',
@@ -38,7 +38,8 @@ __all__ = [ 'CONFIG', 'SB_INFO', 'VERSION', 'PROG_NAME', 'ID_NEW', 'ID_OPEN',
             'ID_MACRO_PLAY', 'ID_ABOUT', 'ID_HOMEPAGE', 'ID_CONTACT',
             'ID_COMMAND_BAR', 'ID_DOCUMENTATION', 'ID_COMMAND',
             'ID_CLOSE_WINDOW', 'ID_USE_SOFTTABS', 'ID_DUP_LINE', 'ID_TRANSLATE',
-            'I18N_PAGE', 'ID_GOTO_MBRACE', 'ID_HLCARET_LINE', 'ID_SHOW_SB'
+            'I18N_PAGE', 'ID_GOTO_MBRACE', 'ID_HLCARET_LINE', 'ID_SHOW_SB',
+            'ID_REVERT_FILE', 'ID_RELOAD_ENC', 'ID_FIND_NEXT'
 ]
 
 #---- Project Info ----#
@@ -67,6 +68,7 @@ if wx.VERSION >= (2, 9, 0, 0, ''):
 #---- Configuration Locations ----#
 # Values set when main loads
 CONFIG = {
+          'CONFIG_BASE' : None, # Set if config base is in nonstandard location
           'CACHE_DIR'   : "",   # Holds temp data about documents
           'PROFILE_DIR' : "",   # User Profile Directory
           'PLUGIN_DIR'  : "",   # User Plugin Dir
@@ -79,6 +81,7 @@ CONFIG = {
 }
 
 DEBUG = False
+SINGLE = True
 
 #---- Object ID's ----#
 # File Menu IDs
@@ -92,6 +95,8 @@ ID_CLOSE_WINDOW  = wx.NewId()
 ID_SAVE          = wx.ID_SAVE
 ID_SAVEAS        = wx.ID_SAVEAS
 ID_SAVEALL       = wx.NewId()
+ID_REVERT_FILE   = wx.ID_REVERT_TO_SAVED
+ID_RELOAD_ENC    = wx.NewId()
 ID_SAVE_PROFILE  = wx.NewId()
 ID_LOAD_PROFILE  = wx.NewId()
 ID_PRINT         = wx.ID_PRINT
@@ -119,6 +124,7 @@ ID_DUP_LINE      = wx.NewId()
 ID_JOIN_LINES    = wx.NewId()
 ID_TRANSPOSE     = wx.NewId()
 ID_FIND          = wx.ID_FIND
+ID_FIND_NEXT     = wx.NewId()
 ID_FIND_REPLACE  = wx.ID_REPLACE
 # Using the system ids automatically disables the menus items
 # when the dialog is open which is not wanted
@@ -135,6 +141,7 @@ ID_PREF_CHKMOD   = wx.NewId()
 ID_PREF_CHKUPDATE = wx.NewId()
 ID_PREF_DLEXER   = wx.NewId()
 ID_PREF_EDGE     = wx.NewId()
+ID_PREF_ENCODING = wx.NewId()
 ID_PREF_SYNTHEME = wx.NewId()
 ID_PREF_TABS     = wx.NewId()
 ID_PREF_UNINDENT = wx.NewId()
@@ -146,6 +153,7 @@ ID_PREF_WPOS     = wx.NewId()
 ID_PREF_ICON     = wx.NewId()
 ID_PREF_ICONSZ   = wx.NewId()
 ID_PREF_MODE     = wx.NewId()
+ID_PREF_TABICON  = wx.NewId()
 ID_PRINT_MODE    = wx.NewId()
 ID_TRANSPARENCY  = wx.NewId()
 ID_PREF_SPOS     = wx.NewId()
@@ -230,16 +238,20 @@ ID_CDROM             = wx.NewId()
 ID_COMMAND_LINE_OPEN = wx.NewId()
 ID_COMMAND_BAR       = wx.NewId()
 ID_COMPUTER          = wx.NewId()
+ID_COPY_PATH         = wx.NewId()
 ID_DELETE            = wx.NewId()
 ID_DOCPROP           = wx.NewId()
 ID_DOWN              = wx.ID_DOWN
 ID_DOWNLOAD_DLG      = wx.NewId()
 ID_FILE              = wx.ID_FILE
+ID_FIND_RESULTS      = wx.NewId()
 ID_FLOPPY            = wx.NewId()
 ID_FOLDER            = wx.NewId()
 ID_HARDDISK          = wx.NewId()
 ID_KEY_PROFILES      = wx.NewId()
+ID_LOGGER            = wx.NewId()
 ID_PACKAGE           = wx.NewId()
+ID_PYSHELL           = wx.NewId()
 ID_REFRESH           = wx.ID_REFRESH
 ID_REPORTER          = wx.NewId()
 ID_STOP              = wx.ID_STOP
@@ -252,7 +264,17 @@ ID_WEB               = wx.NewId()
 # Statusbar IDs
 SB_INFO          = 0
 SB_BUFF          = 1
-SB_ROWCOL        = 2
+SB_LEXER         = 2
+SB_READONLY      = 3
+SB_ENCODING      = 4
+SB_ROWCOL        = 5
+
+# Print Mode Identifiers
+PRINT_BLACK_WHITE = 0
+PRINT_COLOR_WHITE = 1
+PRINT_COLOR_DEF   = 2
+PRINT_INVERT      = 3
+PRINT_NORMAL      = 4
 
 #---- Objects ----#
 
@@ -277,9 +299,11 @@ ID_2_PROF = {
              ID_REPORTER          : 'REPORTER',
              ID_SYNTAX            : 'SYNTAX',
              ID_PREF_DLEXER       : 'DEFAULT_LEX',
+             ID_PREF_ENCODING     : 'ENCODING',
              ID_PREF_SYNTHEME     : 'SYNTHEME',
              ID_PREF_TABS         : 'USETABS',
              ID_PREF_UNINDENT     : 'BSUNINDENT',
+             ID_PREF_TABICON      : 'TABICONS',
              ID_PREF_TABW         : 'TABWIDTH',
              ID_PREF_INDENTW      : 'INDENTWIDTH',
              ID_SESSION           : 'SAVE_SESSION',
@@ -305,4 +329,5 @@ ID_2_PROF = {
 
 # Default Plugins
 DEFAULT_PLUGINS = ("generator.Html", "generator.LaTeX", "generator.Rtf",
-                   "iface.Shelf", "ed_theme.TangoTheme", "ed_log.EdLogViewer")
+                   "iface.Shelf", "ed_theme.TangoTheme", "ed_log.EdLogViewer",
+                   "ed_search.EdFindResults")
