@@ -16,11 +16,11 @@ them accordingly. In the future other key handlers may be added.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_keyh.py 54287 2008-06-19 23:25:40Z CJP $"
-__revision__ = "$Revision: 54287 $"
+__svnid__ = "$Id: ed_keyh.py 55436 2008-09-03 17:03:57Z CJP $"
+__revision__ = "$Revision: 55436 $"
 
 #-------------------------------------------------------------------------#
-# Dependencies
+# Imports
 import re
 import wx, wx.stc
 
@@ -28,8 +28,9 @@ import wx, wx.stc
 import ed_event
 import ed_glob
 import ed_stc
+import ed_cmdbar
 
-# Vi command patterns
+# Vi command regex patterns
 VI_DCMD_RIGHT = '[bBcdeEGhHlLMwWy|{}$<>]'
 VI_DOUBLE_P1 = re.compile('[cdy<>][0-9]*' + VI_DCMD_RIGHT)
 VI_DOUBLE_P2 = re.compile('[0-9]*[cdy<>]' + VI_DCMD_RIGHT)
@@ -38,6 +39,10 @@ VI_GCMDS = re.compile('g[fg]')
 NUM_PAT = re.compile('[0-9]*')
 
 #-------------------------------------------------------------------------#
+# Use this base class to derive any new keyhandlers from. The keyhandler is
+# called upon by the active buffer when ever a key press event happens. The
+# handler then has the responsibility of deciding what to do with the key.
+#
 class KeyHandler:
     """KeyHandler base class"""
     def __init__(self, stc):
@@ -56,15 +61,35 @@ class KeyHandler:
         """
         return u'NULL'
 
-    def ProcessKey(self, key_code):
-        """Process the key and return True if it was processed and
-        false if it was not.
+    def PreProcessKey(self, key_code, ctrldown=False,
+                      cmddown=False, shiftdown=False, altdown=False):
+        """Pre process any keys before they get to the char handler
         @param key_code: Raw keycode
+        @keyword ctrldown: Is the control key down
+        @keyword cmddown: Is the Command key down (Mac osx)
+        @keyword shiftdown: Is the Shift key down
+        @keyword altdown: Is the Alt key down
+        @return: bool
+
+        """
+        return False
+
+    def ProcessKey(self, key_code, ctrldown=False,
+                   cmddown=False, shiftdown=False, altdown=False):
+        """Process the key and return True if it was processed and
+        false if it was not. The key is recieved at EVT_CHAR.
+        @param key_code: Raw keycode
+        @keyword ctrldown: Is the control key down
+        @keyword cmddown: Is the Command key down (Mac osx)
+        @keyword shiftdown: Is the Shift key down
+        @keyword altdown: Is the Alt key down
+        @return: bool
 
         """
         return False
 
 #-------------------------------------------------------------------------#
+
 class ViKeyHandler(KeyHandler):
     """Defines a key handler for Vi emulation
     @summary: Handles keypresses according to Vi emulation.
@@ -78,10 +103,15 @@ class ViKeyHandler(KeyHandler):
 
     def __init__(self, stc):
         KeyHandler.__init__(self, stc)
-        self.SetMode(ViKeyHandler.INSERT)
+
+        # Attributes
+        self.mode = 0
         self.last = u''
         self.cmdcache = u''
 
+        # Use Insert mode by default
+        self.SetMode(ViKeyHandler.INSERT)
+
     def ClearMode(self):
         """Clear the mode back to default input mode"""
         self.stc.SetCaretWidth(1)
@@ -105,33 +135,71 @@ class ViKeyHandler(KeyHandler):
         elif self.mode == ViKeyHandler.INSERT:
             self.stc.SetCaretWidth(1)
             msg = u'INSERT'
+
+        # Update status bar
         evt = ed_event.StatusEvent(ed_event.edEVT_STATUS, self.stc.GetId(),
                                    msg, ed_glob.SB_BUFF)
         wx.PostEvent(self.stc.GetTopLevelParent(), evt)
 
-    def ProcessKey(self, key_code):
+    def PreProcessKey(self, key_code, ctrldown=False,
+                      cmddown=False, shiftdown=False, altdown=False):
+        """Pre process any keys before they get to the char handler
+        @param key_code: Raw keycode
+        @keyword ctrldown: Is the control key down
+        @keyword cmddown: Is the Command key down (Mac osx)
+        @keyword shiftdown: Is the Shift key down
+        @keyword altdown: Is the Alt key down
+        @return: bool
+
+        """
+        if not shiftdown and key_code == wx.WXK_ESCAPE:
+            # If Vi emulation is active go into Normal mode and
+            # pass the key event to the char handler by not processing
+            # the key.
+            self.SetMode(ViKeyHandler.NORMAL)
+            return False
+        elif (ctrldown or cmddown) and key_code == ord('['):
+            self.SetMode(ViKeyHandler.NORMAL)
+            return True
+        else:
+            return False
+
+    def ProcessKey(self, key_code, ctrldown=False,
+                   cmddown=False, shiftdown=False, altdown=False):
         """Processes vi commands
+        @param key_code: Raw key code
+        @keyword cmddown: Command/Ctrl key is down
+        @keyword shiftdown: Shift Key is down
+        @keyword altdown : Alt key is down
         @todo: complete rewrite, this was initially intended as a quick hack
                put together for testing but now has implemented everything.
 
         """
-        if self.mode == ViKeyHandler.INSERT:
+        if self.mode == ViKeyHandler.INSERT or ctrldown or cmddown or altdown:
             return False
+
+        # Add key to cache
         self.cmdcache = self.cmdcache + unichr(key_code)
 
+        # Don't process key if the cache is empty
         if not len(self.cmdcache):
             return False
 
+        # Check for repeat last action command
         if self.cmdcache != u'.':
             cmd = self.cmdcache
         else:
             cmd = self.last
+
+        # Gather common needed data
         cpos = self.stc.GetCurrentPos()
         cline = self.stc.LineFromPosition(cpos)
         mw = self.stc.GetTopLevelParent()
+
+        # Check for change from NORMAL mode to Command mode
         if u':' in cmd:
             self.cmdcache = u''
-            mw.ShowCommandCtrl()
+            mw.ShowCommandCtrl(ed_cmdbar.ID_CMD_CTRL)
 
         # Single key commands
         if len(cmd) == 1 and (cmd in 'AHILmM0^$nia/?:'):
@@ -156,15 +224,16 @@ class ViKeyHandler(KeyHandler):
                 self.stc.GotoPos(cpos + 1)
             elif cmd in u'/?':
                 if mw is not None:
-                    evt = wx.MenuEvent(wx.wxEVT_COMMAND_MENU_SELECTED,
-                                       ed_glob.ID_QUICK_FIND)
-                    wx.PostEvent(mw, evt)
+                    mw.ShowCommandCtrl(ed_cmdbar.ID_SEARCH_CTRL)
 
+            # Return to insert mode
             if cmd in u'aAiI':
                 self.SetMode(ViKeyHandler.INSERT)
 
+            # Save last command and clear cache
             self.last = cmd
             self.cmdcache = u''
+
         # Repeatable 1 key commands
         elif re.match(VI_SINGLE_REPEAT, cmd):
             rcmd = cmd[-1]
@@ -198,13 +267,17 @@ class ViKeyHandler(KeyHandler):
                        u'}' : self.stc.ParaDown,
                        u'~' : self.stc.InvertCase }
 
+            # Command is a Put command
             if rcmd in u'pP':
                 success = False
                 newline = False
+                # Check the clipboard
                 if wx.TheClipboard.Open():
                     td = wx.TextDataObject()
                     success = wx.TheClipboard.GetData(td)
                     wx.TheClipboard.Close()
+
+                # We got the text from the clipboard so put it in the buffer
                 if success:
                     text = td.GetText()
                     if text[-1] == '\n':
@@ -240,6 +313,7 @@ class ViKeyHandler(KeyHandler):
             elif rcmd == u'O':
                 kargs['before'] = True
 
+            # Start an undo action so all changes can be rolled back at once
             self.stc.BeginUndoAction()
             if rcmd in u'CD': # Cut line right
                 self.stc.SetSelection(cpos,
@@ -252,14 +326,18 @@ class ViKeyHandler(KeyHandler):
                 self.stc.SetTargetEnd(self.stc.PositionFromLine(cline + repeat - 1))
                 self.stc.LinesJoin()
             elif rcmd == u'G':
+                # Goto line or to end depending on context
                 if repeat == 1 and '1' not in cmd:
                     repeat = self.stc.GetLineCount()
                 self.stc.GotoLine(repeat - 1)
             elif rcmd == u'+':
+                # Goto indent position at x lines from current line
                 self.stc.GotoIndentPos(cline + repeat)
             elif rcmd == u'-':
+                # Goto indent position at x lines before current line
                 self.stc.GotoIndentPos(cline - repeat)
             elif rcmd == u'|':
+                # Goto column number at current line
                 self.stc.GotoColumn(repeat - 1)
             else:
                 if not cmd_map.has_key(rcmd):
@@ -267,6 +345,8 @@ class ViKeyHandler(KeyHandler):
                 run = cmd_map[rcmd]
                 for count in xrange(repeat):
                     run(*args, **kargs)
+
+            # Do a Put command that is to be repeated
             if rcmd == u'p':
                 if newline:
                     self.stc.GotoIndentPos(cline + repeat)
@@ -282,16 +362,23 @@ class ViKeyHandler(KeyHandler):
             elif rcmd in u'CoOs':
                 self.SetMode(ViKeyHandler.INSERT)
             self.stc.EndUndoAction()
+
+            # Save this action and clear command cache
             self.last = cmd
             self.cmdcache = u''
+
         # 2 key commands
         elif re.match(VI_DOUBLE_P1, cmd) or \
              re.match(VI_DOUBLE_P2, cmd) or \
              re.match(re.compile('[cdy]0'), cmd):
+
+            # Handle repetitions
             if re.match(re.compile('[cdy]0'), cmd):
                 rcmd = cmd
             else:
                 rcmd = re.sub(NUM_PAT, u'', cmd)
+
+            # Get the number of repetitions from the command string
             repeat = re.subn(re.compile(VI_DCMD_RIGHT), u'', cmd, 2)[0]
             if repeat == u'':
                 repeat = 1
@@ -317,14 +404,18 @@ class ViKeyHandler(KeyHandler):
                          u'{' : self.stc.ParaUpExtend,
                          u'}' : self.stc.ParaDownExtend}
 
+                # Select to end of line
                 if u'$' in rcmd:
                     pos = self.stc.GetLineEndPosition(cline + repeat - \
                                                   len(self.stc.GetEOLChar()))
                     self.stc.SetCurrentPos(pos)
                 elif u'G' in rcmd:
-                    if repeat == 0: # invalid cmd
+                    # Check if command is valid
+                    if repeat == 0:
                         self.cmdcache = u''
                         return True
+
+                    # Select to end of file
                     if repeat == 1 and u'1' not in cmd: # Default eof
                         self.stc.SetAnchor(self.stc.GetLineEndPosition(cline - 1))
                         repeat = self.stc.GetLength()
@@ -347,6 +438,7 @@ class ViKeyHandler(KeyHandler):
                     lline = self.stc.GetLastVisibleLine()
 
                     if u'M' in rcmd:
+                        # Get the line in middle of visible lines
                         repeat = self.stc.GetMiddleVisibleLine() + 1
                     elif fline + repeat > lline:
                         repeat = lline
@@ -354,9 +446,11 @@ class ViKeyHandler(KeyHandler):
                         repeat = fline + repeat
 
                     if repeat > cline:
+                        # Move selection lines after middle line
                         self.stc.SetAnchor(self.stc.PositionFromLine(cline))
                         self.stc.SetCurrentPos(self.stc.PositionFromLine(repeat))
                     else:
+                        # Move select lines before middle line
                         self.stc.SetAnchor(self.stc.PositionFromLine(repeat - 1))
                         self.stc.SetCurrentPos(self.stc.PositionFromLine(cline + 1))
                 elif u'L' in rcmd:
@@ -384,27 +478,36 @@ class ViKeyHandler(KeyHandler):
                     for x in xrange(repeat):
                         doit()
 
+            # Begin Undo action so all changes can be rolled back in one undo
             self.stc.BeginUndoAction()
             if re.match(re.compile('c|c' + VI_DCMD_RIGHT), rcmd):
+                # Cut to selection and enter insert mode
                 if rcmd == u'cc':
                     self.stc.SetSelectionEnd(self.stc.GetSelectionEnd() - \
                                          len(self.stc.GetEOLChar()))
                 self.stc.Cut()
                 self.SetMode(ViKeyHandler.INSERT)
             elif re.match(re.compile('d|d' + VI_DCMD_RIGHT), rcmd):
+                # Cut selection and stay in normal mode
                 self.stc.Cut()
             elif re.match(re.compile('y|y' + VI_DCMD_RIGHT), rcmd):
+                # Copy selection
                 self.stc.Copy()
                 self.stc.GotoPos(cpos)
             elif rcmd == u'<<':
+                # Unindent selection
                 self.stc.BackTab()
             elif rcmd == u'>>':
+                # Indent selection
                 self.stc.Tab()
             else:
                 pass
             self.stc.EndUndoAction()
+
             if rcmd in '<<>>' or rcmd[-1] == u'G':
                 self.stc.GotoIndentPos(cline)
+
+            # Save the completed command and clear cache
             self.last = cmd
             self.cmdcache = u''
         elif re.match(VI_GCMDS, cmd):
