@@ -14,24 +14,40 @@ AUTHOR: Cody Precord
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: python.py 52852 2008-03-27 13:45:40Z CJP $"
-__revision__ = "$Revision: 52852 $"
+__svnid__ = "$Id: python.py 55002 2008-08-07 02:03:50Z CJP $"
+__revision__ = "$Revision: 55002 $"
 
 #-----------------------------------------------------------------------------#
 # Dependancies
 import keyword
+import inspect
 
 #-----------------------------------------------------------------------------#
 
 #---- Keyword Specifications ----#
 
+# Indenter keywords
+INDENT_KW = u"def if elif else for while class try except finally"
+
 # Python Keywords
 KEYWORDS = keyword.kwlist
 KEYWORDS.extend(['True', 'False', 'None', 'self'])
-PY_KW = (0, " ".join(KEYWORDS))
+PY_KW = (0, u" ".join(KEYWORDS))
 
-# Highlighted Identifiers
-PY_ID = (1, "")
+# Highlighted builtins
+# TODO: find out why treating __builtins__ like a module here does
+#       not work. It works if 'print __builtins__' but not with
+#       using the dir() method. When this is done it just inspected
+#       as a dict.
+try:
+    BUILTINS = [b for b in __builtins__
+                if inspect.isbuiltin(__builtins__.get(b, None))]
+except:
+    BUILTINS = list()
+BUILTINS.extend(['int', 'hex', 'long', 'float'])
+BUILTINS = list(set(BUILTINS))
+
+PY_BIN = (1, u" ".join(sorted(BUILTINS)))
 
 #---- Syntax Style Specs ----#
 SYNTAX_ITEMS = [ ('STC_P_DEFAULT', 'default_style'),
@@ -49,7 +65,7 @@ SYNTAX_ITEMS = [ ('STC_P_DEFAULT', 'default_style'),
                  ('STC_P_TRIPLE', 'string_style'),
                  ('STC_P_TRIPLEDOUBLE', 'string_style'),
                  ('STC_P_WORD', 'keyword_style'),
-                 ('STC_P_WORD2', 'default_style')]
+                 ('STC_P_WORD2', 'userkw_style')]
 
 #---- Extra Properties ----#
 FOLD = ("fold", "1")
@@ -63,7 +79,7 @@ def Keywords(lang_id=0):
     @param lang_id: used to select specific subset of keywords
 
     """
-    return [PY_KW]
+    return [PY_KW, PY_BIN]
 
 def SyntaxSpec(lang_id=0):
     """Syntax Specifications
@@ -86,6 +102,39 @@ def CommentPattern(lang_id=0):
     """
     return [u'#']
 
+def AutoIndenter(stc, pos, ichar):
+    """Auto indent python code. uses \n the text buffer will
+    handle any eol character formatting.
+    @param stc: EditraStyledTextCtrl
+    @param pos: current carat position
+    @param ichar: Indentation character
+    @return: string
+
+    """
+    rtxt = u''
+    line = stc.GetCurrentLine()
+    text = stc.GetTextRange(stc.PositionFromLine(line), pos)
+
+    if text.isspace():
+        rtxt = '\n'
+    else:
+        indent = stc.GetLineIndentation(line)
+        if ichar == u"\t":
+            tabw = stc.GetTabWidth()
+        else:
+            tabw = stc.GetIndent()
+
+        i_space = indent / tabw
+        ndent = u"\n" + ichar * i_space
+        rtxt = ndent + ((indent - (tabw * i_space)) * u' ')
+
+    tokens = [token for token in text.strip().split() if len(token)]
+    if len(tokens) and tokens[-1].endswith(":"):
+        if tokens[0].rstrip(u":") in INDENT_KW:
+            rtxt += ichar
+
+    return rtxt
+
 #---- End Required Module Functions ----#
 
 #---- Syntax Modules Internal Functions ----#
