@@ -34,8 +34,8 @@ objects such as the Extension Register.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: syntax.py 52978 2008-04-02 10:57:35Z CJP $"
-__revision__ = "$Revision: 52978 $"
+__svnid__ = "$Id: syntax.py 55738 2008-09-19 16:40:49Z CJP $"
+__revision__ = "$Revision: 55738 $"
 
 #-----------------------------------------------------------------------------#
 # Dependencies
@@ -58,6 +58,8 @@ SYNSPEC    = 2    # Highligter specs
 PROPERTIES = 3    # Extra Properties
 LANGUAGE   = 4    # Language ID
 COMMENT    = 5    # Gets the comment characters pattern
+CLEXER     = 6    # Container Lexer Styler Method
+INDENTER   = 7    # Auto-indenter method
 
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
@@ -148,7 +150,7 @@ class SyntaxMgr(object):
             try:
                 self._loaded[modname] = __import__(modname, globals(), 
                                                    locals(), [''])
-            except ImportError:
+            except ImportError, msg:
                 return False
         return True
 
@@ -199,8 +201,13 @@ class SyntaxMgr(object):
         syn_data[PROPERTIES] = mod.Properties(lex_cfg[LANG_ID])
         syn_data[LANGUAGE] = lex_cfg[LANG_ID]
         syn_data[COMMENT] = mod.CommentPattern(lex_cfg[LANG_ID])
-        return syn_data
+        if syn_data[LEXER] == wx.stc.STC_LEX_CONTAINER:
+            syn_data[CLEXER] = mod.StyleText
+        else:
+            syn_data[CLEXER] = None
+        syn_data[INDENTER] = getattr(mod, 'AutoIndenter', None)
 
+        return syn_data
 
 #-----------------------------------------------------------------------------#
 
@@ -215,7 +222,7 @@ def GenLexerMenu():
     for key in synglob.LANG_MAP:
         f_types[key] = synglob.LANG_MAP[key][LANG_ID]
     f_order = list(f_types)
-    f_order.sort()
+    f_order.sort(NoCaseCmp)
 
     for lang in f_order:
         lex_menu.Append(f_types[lang], lang, 
@@ -239,7 +246,7 @@ def GenFileFilters():
     for key in f_dict:
         tmp = u" (%s)|%s|" % (f_dict[key][1:], f_dict[key][1:])
         filters.append(key + tmp)
-    filters.sort()
+    filters.sort(NoCaseCmp)
     filters.insert(0, u"All Files (*)|*|")
     filters[-1] = filters[-1][:-1] # IMPORTANT trim last '|' from item in list
     return filters
@@ -285,6 +292,20 @@ def GetExtFromId(ext_id):
     ftype = synglob.GetDescriptionFromId(ext_id)
     return extreg[ftype][0]
 
+def GetFtypeDisplayName(lang_id):
+    """Get the file type display string for the given lang_id
+    @param lang_id: ID_LANG_*
+    @todo: make file types translatable
+
+    """
+    for item in dir(synglob):
+        if item.startswith("ID_LANG"):
+            if getattr(synglob, item) == lang_id:
+                return getattr(synglob, item[3:], u"Plain Text")
+    else:
+        return u"Plain Text"
+
+
 def GetIdFromExt(ext):
     """Get the language id from the given file extension
     @param ext: file extension (no dot)
@@ -306,3 +327,14 @@ def GetTypeFromExt(ext):
 
     """
     return ExtensionRegister().FileTypeFromExt(ext)
+
+#-----------------------------------------------------------------------------#
+# Utility
+def NoCaseCmp(x, y):
+    """Case insensitive sort method"""
+    if x.lower() < y.lower():
+        return -1
+    elif x.lower() > y.lower():
+        return 1
+    else:
+        return 0
